@@ -18,11 +18,15 @@ from scipy.optimize import leastsq
 from scipy.signal import blackman
 from sklearn.utils import gen_batches
 
+from sidpy.proc.comp_utils import get_available_memory
+from sidpy.hdf.hdf_utils import get_h5_obj_refs, link_h5_objects_as_attrs
+
 from pyUSID import USIDataset
-from pyUSID.io.hdf_utils import get_h5_obj_refs, copy_attributes, link_h5_objects_as_attrs, find_results_groups, \
+from pyUSID.io.hdf_utils import copy_attributes, find_results_groups, \
     link_as_main, check_for_old
-from pyUSID.processing.comp_utils import get_available_memory
-from pyUSID.io.write_utils import make_indices_matrix, get_aux_dset_slicing, INDICES_DTYPE, VALUES_DTYPE, calc_chunks
+from pyUSID.io.write_utils import make_indices_matrix, get_aux_dset_slicing, \
+    INDICES_DTYPE, VALUES_DTYPE, calc_chunks
+
 from ..io.hdf_writer import HDFwriter
 from ..io.virtual_data import VirtualGroup, VirtualDataset
 from .svd_utils import get_component_slice
